/* align form elements to the left */
.esSearchForm form.searchPage {
    text-align: left;
}

.esSearchForm .searchResults em {
  background-color: #FFFFAA;
}
.esSearchForm div.esSearchOptions {
  text-align: left;
}
.esSearchForm .esResultHeader,
.esSearchForm #esSearchLocation,
.esSearchForm .searchResults,
.esSearchForm .emptySearchResults,
.esSearchForm div.esSearchOptions,
.esSearchForm .listingBar {
  display: none;
}

.esSearchForm div.esSearchOptions label {
    text-transform: capitalize;
}

.esSearchForm .searchResults dt.contenttype-file {
    padding-left : 20px;
    background: url("contenttypes-sprite.png") no-repeat scroll 0px 4px transparent;
    line-height : 2em;
}

.esSearchForm img {
    padding-bottom : 0.2em;
}

.esResultHeader {
    height: 20px;
}

.esSearchForm #esSearchSummaryBox {
    font: 1em/1.5em "Lucida Grande",Verdana,Arial,sans-serif !important;
    text-align : left;
    margin: 1em 0 1.2em;
    border: 1px solid #ddd;
    padding: 3px 10px;
    height: 54px;
}

.esSearchForm #esSearchSummaryBox p {
    margin-bottom: 0;
}

.esSearchForm #esSearchSummaryBox p.esSearchOptions {
    padding-bottom: 31px;
    background: url(arrowDown.png) 100px center no-repeat;
}

#content .esSearchForm p.esSearchOptions a {
    border: 0;
}
